# !/usr/bin/ksh
# next line starts wish \
exec wish "$0" "$@"

# -------------------------------------------------------------------------
# (c) 2019, Johann Oberdorfer - Engineering Support | CAD | Software
#     johann.oberdorfer [at] gmail.com
#     www.johann-oberdorfer.eu
# -------------------------------------------------------------------------
# This source file is distributed under the BSD license.
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#   See the BSD License for more details.
# -------------------------------------------------------------------------
#
# Revision history:
# 19-08-04: Hans, Initial release
# -------------------------------------------------------------------------


if {[catch {
	package require starkit
	if {[starkit::startup] eq "sourced"} { return } }]} {

	namespace eval ::starkit {
		if {[set this [info script]] == ""} {
			set this [info nameofexecutable]
		}
		variable topdir [file normalize [file dirname $this]]
	}
}

if {[set ver $::tcl_version] < 8.6} {
	wm withdraw .
	tk_messageBox \
		-title "Installation error:" \
		-message "Tcl/tk version: \"$ver\" is too low, install V8.6 and try again!" \
		-type ok -icon "error"
	exit 0
}

switch -exact [tk windowingsystem] {
	"aqua" {
		set libdir "libbin_osx"
	}
	"win32" -
	"windows" {
		set libdir "libbin"
	}
	"x11" -
	default {
		wm withdraw .
		tk_messageBox \
		-title "Installation error:" \
		-message \
"Sorry, your platform: $plat you are currently working on
is not supported (due to missing kit file binaries)!
Please edit [info script] and try again!" \
		-icon "error"
		exit 0
	}
}


set auto_path [linsert $auto_path 0 [file join $::starkit::topdir "lib"]]
set auto_path [linsert $auto_path 0 [file join $::starkit::topdir $libdir]]
set auto_path [linsert $auto_path 0 [file join $::starkit::topdir "app_images"]]

package require Tk
package require TclOO

# what packages do we need:

package require treectrl
package require Img
package require -exact Tkhtml 3.0
package require twapi

package require twapi_clipboard
# package require img::bmp

package require base64
package require fileutil
package require inifile
package require savedefault
package require screenshot

package require html3widget
package require Markdown
package require scrolledwidget
package require textutil
package require infowindow


package require app_images


set dev_mode 0
if { $dev_mode } {
	catch {
		console show
		console eval {wm protocol . WM_DELETE_WINDOW {exit 0}}
	}
}

array set ::app {
	TITLE "ScreenShot"
	VERSION "V0.1"
}


# source app code ...

source [file join $::starkit::topdir [file join "app" "cleanup_temp.tcl"]]
source [file join $::starkit::topdir [file join "app" "screenshot_about.tcl"]]
source [file join $::starkit::topdir [file join "app" "screenshot_app.tcl"]]


# -----------------------------------
# o.k. let's start the application...
# -----------------------------------
::screenshot_app::main
# -----------------------------------
